@@ -4,13 +4,32 @@ import subprocess
 import yaml
 import pty
 import re
+import logging
 from typing import List, Dict
+from .logging_utils import CustomFormatter
 
+logger = logging.getLogger()
+ch = logging.StreamHandler()
+ch.setLevel(logging.DEBUG)
+ch.setFormatter(CustomFormatter())
+logger.addHandler(ch)
+if int(os.environ.get('BARN_DEBUG', 0)) == 1:
+    logger.setLevel(logging.DEBUG)
+else:
+    logger.setLevel(logging.WARNING)
 
 class IndentedYamlDumper(yaml.Dumper):
     def increase_indent(self, flow=False, indentless=False):
         return super(IndentedYamlDumper, self).increase_indent(flow, False)
     
+def str_presenter(dumper, data):
+  if len(data.splitlines()) > 1:  # check for multiline string
+    return dumper.represent_scalar('tag:yaml.org,2002:str', data, style='|')
+  return dumper.represent_scalar('tag:yaml.org,2002:str', data)
+
+yaml.add_representer(str, str_presenter)
+# to use with safe_dump:
+yaml.representer.SafeRepresenter.add_representer(str, str_presenter)
 
 class Context:
     def __init__(self, root_dir: Path=None, is_initialized=False):
@@ -24,6 +43,10 @@ class Context:
 
         self.project_yaml_path = self.root_dir / "project.yaml"
 
+        self.bash_functions_declarations = {
+
+        }
+
 
 
     def __repr__(self) -> str:
@@ -35,6 +58,7 @@ class Context:
         return repr
 
     def __run_command(self, command, mute=False):
+        logger.debug(f"Executing: {command}")
         master_fd, slave_fd = pty.openpty()
         process = subprocess.Popen(
             command,
@@ -80,6 +104,22 @@ class Context:
 
         return stdout, exit_code
     
+    def setup_bash_function_capabilities(self, name: str, command: str):
+        trailing_sc = " " if command.strip()[-1] != ";" else ""
+        function_declaration = "function "+ name + "() { \n" + command + trailing_sc + '\n}'
+        logger.debug(f"Setting up function: {function_declaration}")
+        _, exit_code = self.__run_command(
+            function_declaration
+        )
+        if exit_code == 0:
+            self.bash_functions_declarations[name] = function_declaration
+        return exit_code
+    
+    def run_function_in_context(self, name: str, args: str):
+        declaration = self.bash_functions_declarations[name]
+        script_to_execute = f"{declaration} && {name} {args}"
+        return self.run_command_in_context(script_to_execute)
+
     def run_command_on_global(self, command: str):
         stdout, exit_code = self.__run_command(
             f'{command}'
@@ -111,8 +151,9 @@ class Context:
     def install_from_project(self):
         requirements = self.get_dependencies_from_project_yaml()
         requirements_command = ' '.join(requirements)
-        self.run_command_in_context(f"pip install {requirements_command}")
+        stdout, exit_code = self.run_command_in_context(f"pip install {requirements_command}")
         self.freeze_lock()
+        return stdout, exit_code
 
     def get_project_config(self):
         # Load the YAML file
@@ -125,7 +166,7 @@ class Context:
         yaml_content = self.get_project_config()
         if is_url:
             new_entry = {
-                package: f"N/A"
+                package: "N/A"
             }
         else:
             new_entry = {
@@ -138,6 +179,7 @@ class Context:
         dependencies: list[any] = yaml_content["dependencies"]
         if new_entry not in dependencies:
             yaml_content["dependencies"].append(new_entry)
+            logger.debug(yaml_content)
             with open(self.project_yaml_path, 'w') as yaml_file:
                 yaml.dump(yaml_content, yaml_file, Dumper=IndentedYamlDumper, default_flow_style=False, indent=2, sort_keys=False)
 
@@ -151,8 +193,7 @@ class Context:
                 break
         
         yaml_content["dependencies"] = dependencies
-        
-            
+
         with open(self.project_yaml_path, 'w') as yaml_file:
             yaml.dump(yaml_content, yaml_file, Dumper=IndentedYamlDumper, default_flow_style=False, indent=2, sort_keys=False)
 
@@ -200,11 +241,11 @@ def find_project_yaml():
             return current_dir / "project.yaml"
         current_dir = current_dir.parent
     
-    
     raise Exception("Barn could not find a project context to use. Are you in a project directory?")
 
 
 
+
 def barn_action(action):
     project_yaml = find_project_yaml()
     root_dir = project_yaml.parent
@@ -214,6 +255,6 @@ def barn_action(action):
         is_initialized=is_initialized
     )
     def wrapper(*args, **kwargs):
-        action(*args, **kwargs, context=context)
+        return action(*args, **kwargs, context=context, logger=logger)
 
     return wrapper
